@@ -12,6 +12,5 @@ HYPERLINK_POOL_HANDLE alloc_hyperlink_pool(void);
 void free_hyperlink_pool(HYPERLINK_POOL_HANDLE);
 void clear_hyperlink_pool(HYPERLINK_POOL_HANDLE);
 hyperlink_id_type get_id_for_hyperlink(Screen*, const char*, const char*);
-hyperlink_id_type remap_hyperlink_ids(Screen *self, hyperlink_id_type *map);
-PyObject* screen_hyperlinks_as_list(Screen *screen);
+PyObject* screen_hyperlinks_as_set(Screen *screen);
 void screen_garbage_collect_hyperlink_pool(Screen *screen);
