@@ -2,10 +2,10 @@
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
 from collections.abc import Collection, Generator, Sequence
-from typing import Any, NamedTuple, Optional, Union
+from typing import Any, NamedTuple, Optional, TypedDict, Union
 
 from kitty.borders import BorderColor
-from kitty.types import Edges, WindowGeometry
+from kitty.types import Edges, WindowGeometry, WindowMapper
 from kitty.typing_compat import EdgeLiteral, WindowType
 from kitty.window_list import WindowGroup, WindowList
 
@@ -17,6 +17,13 @@ class Extent(NamedTuple):
     end: int = 0
 
 
+class SerializedPair(TypedDict, total=False):
+    horizontal: bool  # default to True if absent
+    bias: float  # default to 0.5 if absent
+    one: Union[int, 'SerializedPair']  # default to None if absent
+    two: Union[int, 'SerializedPair']  # default to None if absent
+
+
 class Pair:
 
     def __init__(self, horizontal: bool = True):
@@ -28,6 +35,33 @@ class Pair:
         self.between_borders: list[Edges] = []
         self.first_extent = self.second_extent = Extent()
 
+    def serialize(self) -> SerializedPair:
+        ans: SerializedPair = {}
+        if not self.horizontal:
+            ans['horizontal'] = False
+        if self.bias != 0.5:
+            ans['bias'] = self.bias
+        if self.one is not None:
+            ans['one'] = self.one.serialize() if isinstance(self.one, Pair) else self.one
+        if self.two is not None:
+            ans['two'] = self.two.serialize() if isinstance(self.two, Pair) else self.two
+        return ans
+
+    def unserialize(self, s: SerializedPair, map_window_id: WindowMapper) -> None:
+        self.bias = s.get('bias', 0.5)
+        self.horizontal = s.get('horizontal', True)
+
+        def unserialize(x: int | SerializedPair | None) -> int | Pair | None:
+            if x is None:
+                return None
+            if isinstance(x, int):
+                return map_window_id(x)
+            ans = Pair()
+            ans.unserialize(x, map_window_id)
+            return ans if ans.one or ans.two else None
+        self.one = unserialize(s.get('one'))
+        self.two = unserialize(s.get('two'))
+
     def __repr__(self) -> str:
         return 'Pair(horizontal={}, bias={:.2f}, one={}, two={}, between_borders={})'.format(
                 self.horizontal, self.bias, self.one, self.two, self.between_borders)
@@ -673,19 +707,14 @@ class Splits(Layout):
         return None
 
     def layout_state(self) -> dict[str, Any]:
+        return {'pairs': self.pairs_root.serialize()}
 
-        def add_pair(p: Pair) -> dict[str, Any]:
-            ans: dict[str, Any] = {}
-            ans['horizontal'] = p.horizontal
-            ans['bias'] = p.bias
-            if isinstance(p.one, Pair):
-                ans['one'] = add_pair(p.one)
-            elif p.one is not None:
-                ans['one'] = p.one
-            if isinstance(p.two, Pair):
-                ans['two'] = add_pair(p.two)
-            elif p.two is not None:
-                ans['two'] = p.two
-            return ans
-
-        return {'pairs': add_pair(self.pairs_root)}
+    def set_layout_state(self, layout_state: dict[str, Any], map_group_id: WindowMapper) -> bool:
+        new_root = Pair()
+        new_root.unserialize(layout_state['pairs'], map_group_id)
+        before = frozenset(self.pairs_root.all_window_ids())
+        if before == frozenset(new_root.all_window_ids()):
+            self.pairs_root = new_root
+            self.layout_opts = SplitsLayoutOpts(layout_state['opts'])
+            return True
+        return False
