@@ -27,23 +27,26 @@ log_error(const char *fmt, ...) {
     va_end(ar);
     if (n < 0) return;
     size_t size = 5 * (size_t)n + 8;
-    RAII_ALLOC(char, arena, calloc(size, sizeof(char)));
+    RAII_ALLOC(unsigned char, arena, calloc(size, sizeof(char)));
     if (!arena) return;
     va_start(ar, fmt);
-    n = vsnprintf(arena, size, fmt, ar);
+    n = vsnprintf((char*)arena, size, fmt, ar);
     va_end(ar);
-    char *sanbuf = arena + n + 1;
+    unsigned char *sanbuf = arena + n + 1;
 
     char utf8buf[4];
     START_ALLOW_CASE_RANGE
     size_t j = 0;
-    for (char *x = arena; x < arena + n; x++) {
+    for (unsigned char *x = arena; x < arena + n; x++) {
         switch(*x) {
-            case C0_EXCEPT_NL_SPACE_TAB: {
+            case C0_EXCEPT_NL_SPACE_TAB_DEL: {
                 const uint32_t ch = 0x2400 + *x;
                 const unsigned sz = encode_utf8(ch, utf8buf);
                 for (unsigned c = 0; c < sz; c++, j++) sanbuf[j] = utf8buf[c];
             } break;
+            case 0x7f:
+                sanbuf[j++] = 0xe2; sanbuf[j++] = 0x90; sanbuf[j++] = 0xa1; // U+2421
+                break;
             default:
                 sanbuf[j++] = *x;
                 break;
