@@ -7,9 +7,10 @@ from dataclasses import dataclass, field
 from io import BytesIO
 from typing import TYPE_CHECKING, Any, NoReturn, Optional, Union, cast
 
-from kitty.cli import CompletionSpec, get_defaults_from_seq, parse_args, parse_option_spec
+from kitty.cli import get_defaults_from_seq, parse_args
 from kitty.cli_stub import RCOptions as R
 from kitty.constants import list_kitty_resources, running_in_kitty
+from kitty.simple_cli_definitions import CompletionSpec, parse_option_spec
 from kitty.types import AsyncResponse
 
 if TYPE_CHECKING:
@@ -90,7 +91,7 @@ MATCH_WINDOW_OPTION = '''\
 --match -m
 The window to match. Match specifications are of the form: :italic:`field:query`.
 Where :italic:`field` can be one of: :code:`id`, :code:`title`, :code:`pid`, :code:`cwd`, :code:`cmdline`, :code:`num`,
-:code:`env`, :code:`var`, :code:`state`, :code:`neighbor`, and :code:`recent`.
+:code:`env`, :code:`var`, :code:`state`, :code:`neighbor`, :code:`session` and :code:`recent`.
 :italic:`query` is the expression to match. Expressions can be either a number or a regular expression, and can be
 :ref:`combined using Boolean operators <search_syntax>`.
 
@@ -111,6 +112,11 @@ active window, one being the previously active window and so on.
 The field :code:`neighbor` refers to a neighbor of the active window in the specified direction, which can be:
 :code:`left`, :code:`right`, :code:`top` or :code:`bottom`.
 
+The field :code:`session` matches windows that were created in the specified session.
+Use the expression :code:`^$` to match windows that were not created in a session and
+:code:`.` to match the currently active session and :code:`~` to match either the currently
+active sesison or the last active session when no session is active.
+
 When using the :code:`env` field to match on environment variables, you can specify only the environment variable name
 or a name and value, for example, :code:`env:MY_ENV_VAR=2`.
 
@@ -134,7 +140,7 @@ MATCH_TAB_OPTION = '''\
 --match -m
 The tab to match. Match specifications are of the form: :italic:`field:query`.
 Where :italic:`field` can be one of: :code:`id`, :code:`index`, :code:`title`, :code:`window_id`, :code:`window_title`,
-:code:`pid`, :code:`cwd`, :code:`cmdline` :code:`env`, :code:`var`, :code:`state` and :code:`recent`.
+:code:`pid`, :code:`cwd`, :code:`cmdline` :code:`env`, :code:`var`, :code:`state`, :code:`session` and :code:`recent`.
 :italic:`query` is the expression to match. Expressions can be either a number or a regular expression, and can be
 :ref:`combined using Boolean operators <search_syntax>`.
 
@@ -154,6 +160,11 @@ The :code:`index` number is used to match the nth tab in the currently active OS
 The :code:`recent` number matches recently active tabs in the currently active OS window, with zero being the currently
 active tab, one the previously active tab and so on.
 
+The field :code:`session` matches tabs that were created in the specified session.
+Use the expression :code:`^$` to match windows that were not created in a session and
+:code:`.` to match the currently active session and :code:`~` to match either the currently
+active sesison or the last active session when no session is active.
+
 When using the :code:`env` field to match on environment variables, you can specify only the environment variable name
 or a name and value, for example, :code:`env:MY_ENV_VAR=2`. Tabs containing any window with the specified environment
 variables are matched. Similarly, :code:`var` matches tabs containing any window with the specified user variable.
@@ -381,8 +392,7 @@ class RemoteCommand:
                 raise MatchError(match, 'tabs')
             return tabs
         if window and payload_get('self') in (None, True):
-            q = boss.tab_for_window(window)
-            if q:
+            if q := window.tabref():
                 return [q]
         t = boss.active_tab
         if t:
