@@ -7,8 +7,8 @@ import (
 	"fmt"
 	"strings"
 
-	"kitty/tools/tui/loop"
-	"kitty/tools/wcswidth"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/wcswidth"
 )
 
 type KilledBySignal struct {
@@ -21,7 +21,7 @@ func (self *KilledBySignal) Error() string { return self.Msg }
 var Canceled = errors.New("Canceled by user")
 
 func ReadPassword(prompt string, kill_if_signaled bool) (password string, err error) {
-	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors)
+	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.FullKeyboardProtocol)
 	shadow := ""
 	if err != nil {
 		return
