@@ -5,13 +5,14 @@ package utils
 import (
 	"fmt"
 	"os"
+	"sync"
 )
 
 var _ = fmt.Print
 
 var hostname string = "*"
 
-var Hostname = Once(func() string {
+var Hostname = sync.OnceValue(func() string {
 	h, err := os.Hostname()
 	if err == nil {
 		return h
